@@ -17,7 +17,6 @@
 
 package cds.healpix;
 
-import java.util.Arrays;
 import java.util.EnumMap;
 import java.util.EnumSet;
 
@@ -161,17 +160,17 @@ public final class HealpixNestedFast implements HashComputer, VerticesAndPathCom
     // We change the algo with respect to the cleaner code in HealpixNestedHashComputer.
     // The reasons are:
     // - see class Javadoc (for performances: we introduce redundancy, inline all method calls, ...)
-    // - as we introduce redondancy, we do not have to keep the projection code separated
+    // - as we introduce redundancy, we do not have to keep the projection code separated
     //   - it allows to perform the computations in the 2 canonical base cells (NPC: 0; EQR: 4),
     //     with the y-origin of the Collignon projection set to 0 (instead of 1), and only then 
     //     we consider the base cell number
     // - thus we also removed the base cell lookup table:
-    //   - it saves CPU cache utilisation and lookup
+    //   - it saves CPU cache
     //   - but it introduces 'if' conditions that we tried to avoid: effect should be limited since 
-    //     few risks of branche miss-predictions, except in South/North polar caps (effect should be
-    //     negligeable, except if input positions are random and more or less uniformly distributed
+    //     few risks of branch miss-predictions, except in South/North polar caps (effect should be
+    //     negligible, except if input positions are random and more or less uniformly distributed
     //     in South and North polar caps).
-    // - See HealpixNestedHashComputer code (with matrix lookup) for a completly branch free version
+    // - See HealpixNestedHashComputer code (with matrix lookup) for a completely branch free version
     //   (except for the Cylindrical/Collignon test).
     checkLatitude(latRad);
     final double absLon = fromBits(toBits(lonRad) & BUT_SIGN_BIT_MASK_L);  assert  0 <= absLon && absLon <= 10 * TWO_PI : absLon; // 10 is arbitrary
@@ -181,6 +180,10 @@ public final class HealpixNestedFast implements HashComputer, VerticesAndPathCom
     double x = FOUR_OVER_PI * absLon, y;
     int xfloor = ((int) x);
     x -= (xfloor | 1);                                                     assert -1 <= x && x <= 1;
+    if (lonRad < 0) { // We expect lonRad to be >= 0, so we should rarely enter here
+      x = -x;         // (thus few risks of branch miss-prediction)
+      xfloor = 7 - (xfloor & 7);
+    }
     long d0h;
     int i, j;
     if (absLat <= TRANSITION_LATITUDE) { // Equatorial region (Cylindrical projection)
@@ -511,21 +514,21 @@ public final class HealpixNestedFast implements HashComputer, VerticesAndPathCom
     final int hInD0h = iInD0h + jInD0h - this.modNsideMask;
     // Compute coordinates of the base pixel in the projection plane
     final int d0hBy4Quotient = d0h >> 2;
-        final int d0hMod4 = d0h & 3;
-        final int hD0h = 1 - d0hBy4Quotient;
-        // +1 if the base cell is not equatorial
-        int lD0h = (d0hMod4 << 1) | (hD0h & 1);
-        // Compute let's go
-        double lonC = lInD0h * this.oneOverNside;
-        double latC = hInD0h * this.oneOverNside;
-        // Let's go
-        if ((d0h < 4 && hInD0h > 0) || (d0h > 7 && hInD0h < 0)) { // Polar cap
-          pathAlongCellSidePolarCap(lonC, latC, lD0h, fromVertex, toVertex,
-              isToVertexIncluded, nSegments, pathPoints, fromPathPointsIndex);
-        } else { // Equatorial zone
-          pathAlongCellSideEquatRegion(lonC, latC, lD0h, d0h, hD0h, lInD0h, fromVertex, toVertex,
-              isToVertexIncluded, nSegments, pathPoints, fromPathPointsIndex);
-        }
+    final int d0hMod4 = d0h & 3;
+    final int hD0h = 1 - d0hBy4Quotient;
+    // +1 if the base cell is not equatorial
+    int lD0h = (d0hMod4 << 1) | (hD0h & 1);
+    // Compute let's go
+    double lonC = lInD0h * this.oneOverNside;
+    double latC = hInD0h * this.oneOverNside;
+    // Let's go
+    if ((d0h < 4 && hInD0h > 0) || (d0h > 7 && hInD0h < 0)) { // Polar cap
+      pathAlongCellSidePolarCap(lonC, latC, lD0h, fromVertex, toVertex,
+        isToVertexIncluded, nSegments, pathPoints, fromPathPointsIndex);
+    } else { // Equatorial zone
+      pathAlongCellSideEquatRegion(lonC, latC, lD0h, d0h, hD0h, lInD0h, fromVertex, toVertex,
+          isToVertexIncluded, nSegments, pathPoints, fromPathPointsIndex);
+    }
   }
 
   private void pathAlongCellSidePolarCap(final double lonC, final double latC, final int lD0h,
@@ -539,13 +542,12 @@ public final class HealpixNestedFast implements HashComputer, VerticesAndPathCom
     // Compute stepX and stepY
     final double stepX = (toVertex.timeXOffset(this.oneOverNside) - fromOffsetX) / nSegments;
     final double stepY = (toVertex.timeYOffset(this.oneOverNside) - fromOffsetY) / nSegments;
+    final long signLat = toBits(latC) & SIGN_BIT_MASK_L; // All points of the path are in the same hemisphere,
     for (int i = 0; i < resultSize; i++) {
       final double[] pathPoint = new double[2];
       double lon = lonC + fromOffsetX + i * stepX;
       double lat = latC + fromOffsetY + i * stepY;
-      long signLat = toBits(lat);
-      lat = fromBits(signLat & BUT_SIGN_BIT_MASK_L);
-      signLat &= SIGN_BIT_MASK_L;
+      lat = fromBits(toBits(lat) & BUT_SIGN_BIT_MASK_L);
       lat = 1 - lat;
       lon /= lat;
       lat = fromBits(signLat | toBits(2 * acos(lat * ONE_OVER_SQRT6) - HALF_PI));
